@@ -1,4 +1,4 @@
-/* -*- mode: C; c-file-style: "gnu" -*- */
+/* -*- mode: C; c-file-style: "gnu"; indent-tabs-mode: nil; -*- */
 /* dbus-keyring.c Store secret cookies in your homedir
  *
  * Copyright (C) 2003, 2004  Red Hat Inc.
@@ -201,9 +201,8 @@ _dbus_keyring_lock (DBusKeyring *keyring)
   n_timeouts = 0;
   while (n_timeouts < MAX_LOCK_TIMEOUTS)
     {
-      DBusError error;
+      DBusError error = DBUS_ERROR_INIT;
 
-      dbus_error_init (&error);
       if (_dbus_create_file_exclusively (&keyring->filename_lock,
                                          &error))
         break;
@@ -219,13 +218,11 @@ _dbus_keyring_lock (DBusKeyring *keyring)
 
   if (n_timeouts == MAX_LOCK_TIMEOUTS)
     {
-      DBusError error;
-      
+      DBusError error = DBUS_ERROR_INIT;
+
       _dbus_verbose ("Lock file timed out %d times, assuming stale\n",
                      n_timeouts);
 
-      dbus_error_init (&error);
-
       if (!_dbus_delete_file (&keyring->filename_lock, &error))
         {
           _dbus_verbose ("Couldn't delete old lock file: %s\n",
@@ -250,8 +247,8 @@ _dbus_keyring_lock (DBusKeyring *keyring)
 static void
 _dbus_keyring_unlock (DBusKeyring *keyring)
 {
-  DBusError error;
-  dbus_error_init (&error);
+  DBusError error = DBUS_ERROR_INIT;
+
   if (!_dbus_delete_file (&keyring->filename_lock, &error))
     {
       _dbus_warn ("Failed to delete lock file: %s\n",
