@@ -1,14 +1,18 @@
 /* This is simply a process that segfaults */
+#include <config.h>
+#include <stdlib.h>
 #include <signal.h>
 
-#include <sys/time.h>
+#ifdef HAVE_SETRLIMIT
 #include <sys/resource.h>
+#endif
 
 int
 main (int argc, char **argv)
 {
   char *p;  
 
+#if HAVE_SETRLIMIT
   struct rlimit r = { 0, };
   
   getrlimit (RLIMIT_CORE, &r);
@@ -16,8 +20,8 @@ main (int argc, char **argv)
   setrlimit (RLIMIT_CORE, &r);
   
   raise (SIGSEGV);
-
-  p = 0;
+#endif
+  p = NULL;
   *p = 'a';
   
   return 0;
