@@ -38,16 +38,19 @@ if ! test -e "$DBUS_TOP_BUILDDIR"/bus/dbus-daemon ; then
     die "$DBUS_TOP_BUILDDIR/bus/dbus-daemon does not exist"
 fi
 
-export PATH="$DBUS_TOP_BUILDDIR"/bus:$PATH
-## the libtool script found by the path search should already do this, but
-export LD_LIBRARY_PATH=$DBUS_TOP_BUILDDIR/dbus/.libs:$LD_LIBRARY_PATH
+PATH="$DBUS_TOP_BUILDDIR"/bus:$PATH
+export PATH
 
+## the libtool script found by the path search should already do this, but
+LD_LIBRARY_PATH=$DBUS_TOP_BUILDDIR/dbus/.libs:$LD_LIBRARY_PATH
+export LD_LIBRARY_PATH
 unset DBUS_SESSION_BUS_ADDRESS
 unset DBUS_SESSION_BUS_PID
 
 echo "Running $DBUS_TOP_BUILDDIR/tools/dbus-launch --sh-syntax --config-file=$CONFIG_FILE" >&2
 
-export DBUS_USE_TEST_BINARY=1 
+DBUS_USE_TEST_BINARY=1 
+export DBUS_USE_TEST_BINARY
 eval `$DBUS_TOP_BUILDDIR/tools/dbus-launch --sh-syntax --config-file=$CONFIG_FILE`
 
 if test -z "$DBUS_SESSION_BUS_PID" ; then
